# DistributionModelsPHT

This is a [Julia](https://julialang.org/) implementation of distributions that approximate the distribution of the area of the Poisson polygon or the Crofton polygon as well as the volume and the surface area of the Poisson polyhedron or the Crofton polyhedron by a suitable mixture of generalized gamma distributions (Stacy, 1962).

The implementations of the distributions are all realized as subtypes of the type `Distributions.ContinuousUnivariateDistribution` from the Julia package [Distributions](https://juliastats.org/Distributions.jl/stable/) and provide the usual methods for parameter queries, random number generation and the calculation of distribution function, distribution density and simple statistics such as mean and variance.

## Description
The polygons/polyhedra under consideration are cells associated with the stationary and isotropic _Poisson line/plane tessellation_. Therein the space is divided into polygonal/polyhedral cells by (infinite) lines/planes which exhibit random orientation and location such that the entire system is both stationary and isotropic in 2D/3D space. There is one model parameter 'λ' equal to the mean line length per unit area/mean plane area per unit volume of the underlying Poisson line/plane tessellation. See Schneider & Weil (2008) and Chiu et al. (2013) for the theoretical background.

The _Crofton polygon/polyhedron_ is the _zero cell_ of this tessellation, i.e. the cell that (almost surely) contains the origin.

The _Poisson polygon/polyhedron_ is the so-called _typical cell_ of this tessellaton, which can be understood as the result of a random selection among all cells of the mosaic, in which all cells have the same chance of being selected.

All six distributions 
- AreaPoissonPolygon
- AreaCroftonPolygon
- VolumePoissonPolyhedron
- VolumeCroftonPolyhedron
- SurfacePoissonPolyhedron
- SurfaceCroftonPolyhedron

have a parameter 'λ' (λ > 0) that is inherited from the underlying Poisson line/plane tessellation.

The parameter λ does not behave like a rate parameter itself, but instead λ^2 for Area... and Surface... and λ^3 for Volume... .

The distributions were fitted from very extensive simulation data (Ballani, 2024).

## Usage

```julia
using DistributionModelsPHT
using Distributions
```

### Area of the Poisson polygon

```julia
AreaPoissonPolygon()           # equivalent to AreaPoissonPolygon(1)
AreaPoissonPolygon(1.5)        # approximate distribution of the area of the Poisson polygon for the parameter λ=1.5
params(AreaPoissonPolygon(2))  # get the parameter
cdf(AreaPoissonPolygon(1),1:3) # evaluate the cumulative distribution function at 1, 2, 3
pdf(AreaPoissonPolygon(1),1:3) # evaluate the probability density function at 1, 2, 3
rand(AreaPoissonPolygon(1))    # generate a random number from AreaPoissonPolygon(1)
rand(AreaPoissonPolygon(1),10) # generate a sample of 10 random numbers from AreaPoissonPolygon(1)
```

### Area of the Crofton polygon

```julia
AreaCroftonPolygon()           # equivalent to AreaCroftonPolygon(1)
AreaCroftonPolygon(1.5)        # approximate distribution of the area of the Crofton polygon for the parameter λ=1.5
```

### Volume of the Poisson polyhedron

```julia
VolumePoissonPolyhedron()      # equivalent to VolumePoissonPolyhedron(1)
VolumePoissonPolyhedron(1.5)   # approximate distribution of the volume of the Poisson polyhedron for the parameter λ=1.5
```

### Volume of the Crofton polyhedron

```julia
VolumeCroftonPolyhedron()      # equivalent to VolumeCroftonPolyhedron(1)
VolumeCroftonPolyhedron(1.5)   # approximate distribution of the volume of the Crofton polyhedron for the parameter λ=1.5
```

### Surface area of the Poisson polyhedron

```julia
SurfacePoissonPolyhedron()     # equivalent to SurfacePoissonPolyhedron(1)
SurfacePoissonPolyhedron(1.5)  # approximate distribution of the surface area of the Poisson polyhedron for the parameter λ=1.5
```

### Surface area of the Crofton polyhedron

```julia
SurfaceCroftonPolyhedron()     # equivalent to SurfaceCroftonPolyhedron(1)
SurfaceCroftonPolyhedron(1.5)  # approximate distribution of the surface area of the Crofton polyhedron for the parameter λ=1.5
```

### Mixture weigths and components

Each of the six distributions is a mixture of generalized gamma distributions. The mixture weights can be obtained with the help of `Distributions.probs` and the distribution components with `Distributions.components`.

```julia
probs(AreaPoissonPolygon())
components(AreaPoissonPolygon())
```

## License
This code is free to use under the terms of the MIT license.

## References
- Ballani, F. (2024). Data publication: Simulation results for standard statistics of Poisson and Crofton cells. [https://doi.org/10.14278/rodare.3290](https://doi.org/10.14278/rodare.3290)
- Chiu S. N., Stoyan D., Kendall W. S. and Mecke, J. (2013). Stochastic Geometry and its Applications, 3rd edn. J. Wiley & Sons, Chichester, [https://doi.org/10.1002/9781118658222](https://doi.org/10.1002/9781118658222)
- Schneider, R. and Weil, W. (2008), Stochastic and Integral Geometry, Springer, Berlin. [https://doi.org/10.1007/978-3-540-78859-1](https://doi.org/10.1007/978-3-540-78859-1)
- Stacy, E. W. (1962). A generalization of the gamma distribution. Annals of Mathematical Statistics 33(3):1187–1192. [https://doi.org/10.1214/aoms/1177704481](https://doi.org/10.1214/aoms/1177704481)
