"""
    AreaCroftonPolygon(λ)

An approximation of the distribution of the area of the Crofton polygon with intensity parameter `λ` in terms of a finite mixture of generalized gamma distributions.

```julia
AreaCroftonPolygon()    # `AreaCroftonPolygon`-distribution with unit intensity, i.e. AreaCroftonPolygon(1)
AreaCroftonPolygon(λ)   # `AreaCroftonPolygon`-distribution with intensity λ
```

"""
struct AreaCroftonPolygon{T<:Real} <: Distributions.ContinuousUnivariateDistribution
    λ::T
    AreaCroftonPolygon{T}(λ) where {T} = new{T}(λ)
end

function AreaCroftonPolygon(λ::T; check_args::Bool=true) where {T<:Real}
    Distributions.@check_args AreaCroftonPolygon (λ, λ > zero(λ))
    return AreaCroftonPolygon{T}(λ)
end

AreaCroftonPolygon(λ::Integer; check_args::Bool=true) = AreaCroftonPolygon(float(λ); check_args=check_args)
AreaCroftonPolygon() = AreaCroftonPolygon{Float64}(1.0)

Distributions.@distr_support AreaCroftonPolygon 0.0 Inf

#### Conversions

Distributions.convert(::Type{AreaCroftonPolygon{T}}, λ::S) where {T<:Real, S<:Real} = AreaCroftonPolygon(T(λ))
Base.convert(::Type{AreaCroftonPolygon{T}}, d::AreaCroftonPolygon) where {T<:Real} = AreaCroftonPolygon{T}(T(d.λ))
Base.convert(::Type{AreaCroftonPolygon{T}}, d::AreaCroftonPolygon{T}) where {T<:Real} = d

#### Parameters

Distributions.params(d::AreaCroftonPolygon) = d.λ
Distributions.partype(::AreaCroftonPolygon{T}) where {T} = T

#### Evaluation & Sampling

__AreaC2 = Distributions.MixtureModel( 
               GGamma[GGamma(pars...) for pars in 
                       [ [0.2706845074530123, 1.5, 0.4694056921354835],
                         [0.3026240868403616, 2.0, 0.46588630322281827],
                         [0.34948454672244983, 2.5, 0.47041955117610335],
                         [0.3913645422130834, 3.0, 0.47467273433862334],
                         [0.42788580914370894, 3.5, 0.47828317758565636],
                         [0.458526508844167, 4.0, 0.48113799705334387],
                         [0.4525352988546085, 4.5, 0.4783957513991455],
                         [0.4623425881810003, 5.0, 0.47930071815827924] ]
                     ], 
               [0.07683507643070003, 0.30046578689231634, 0.34266073831954624, 0.19389978457526236, 
                0.06713884315147821, 0.015887884412591775, 0.0027501115445242455, 0.0003617746735807604] )

Base.rand(rng::Distributions.AbstractRNG, d::AreaCroftonPolygon) = Base.rand(rng, __AreaC2) / d.λ^2

Distributions.pdf(d::AreaCroftonPolygon, x::Real) = Distributions.pdf(__AreaC2, d.λ^2 * x) * d.λ^2
Distributions.logpdf(d::AreaCroftonPolygon, x::Real) = Distributions.logpdf(__AreaC2, d.λ^2 * x) + 2 * log(d.λ)
Distributions.cdf(d::AreaCroftonPolygon, x::Real) = Distributions.cdf(__AreaC2, d.λ^2 * x)
Distributions.quantile(d::AreaCroftonPolygon, q::Real) = Distribution.quantile(__AreaC2, q) / d.λ^2

Base.minimum(d::AreaCroftonPolygon) = 0.0
Base.maximum(d::AreaCroftonPolygon) = Inf
Distributions.insupport(d::AreaCroftonPolygon, x::Real) = Distributions.insupport(__AreaC2, x)

Distributions.probs(d::AreaCroftonPolygon) = Distributions.probs(__AreaC2)
Distributions.component_type(d::AreaCroftonPolygon) = Distributions.component_type(__AreaC2)
function Distributions.components(d::AreaCroftonPolygon) 
    map(x -> GGamma(Distributions.scale(x) / d.λ^2, Distributions.shape(x)...), Distributions.components(__AreaC2))
end

#### Statistics

Statistics.mean(d::AreaCroftonPolygon) = Statistics.mean(__AreaC2) / d.λ^2
Statistics.var(d::AreaCroftonPolygon) = Statistics.var(__AreaC2) / d.λ^4
