"""
    AreaPoissonPolygon(λ)

An approximation of the distribution of the area of the Poisson polygon with intensity parameter `λ` in terms of a finite mixture of generalized gamma distributions.

```julia
AreaPoissonPolygon()    # `AreaPoissonPolygon`-distribution with unit intensity, i.e. AreaPoissonPolygon(1)
AreaPoissonPolygon(λ)   # `AreaPoissonPolygon`-distribution with intensity λ
```

"""
struct AreaPoissonPolygon{T<:Real} <: Distributions.ContinuousUnivariateDistribution
    λ::T
    AreaPoissonPolygon{T}(λ) where {T} = new{T}(λ)
end

function AreaPoissonPolygon(λ::T; check_args::Bool=true) where {T<:Real}
    Distributions.@check_args AreaPoissonPolygon (λ, λ > zero(λ))
    return AreaPoissonPolygon{T}(λ)
end

AreaPoissonPolygon(λ::Integer; check_args::Bool=true) = AreaPoissonPolygon(float(λ); check_args=check_args)
AreaPoissonPolygon() = AreaPoissonPolygon{Float64}(1.0)

Distributions.@distr_support AreaPoissonPolygon 0.0 Inf

#### Conversions

Distributions.convert(::Type{AreaPoissonPolygon{T}}, λ::S) where {T<:Real, S<:Real} = AreaPoissonPolygon(T(λ))
Base.convert(::Type{AreaPoissonPolygon{T}}, d::AreaPoissonPolygon) where {T<:Real} = AreaPoissonPolygon{T}(T(d.λ))
Base.convert(::Type{AreaPoissonPolygon{T}}, d::AreaPoissonPolygon{T}) where {T<:Real} = d

#### Parameters

Distributions.params(d::AreaPoissonPolygon) = d.λ
Distributions.partype(::AreaPoissonPolygon{T}) where {T} = T

#### Evaluation & Sampling

__AreaP2 = Distributions.MixtureModel( 
               GGamma[GGamma(pars...) for pars in 
                       [ [0.2709672153873192, 0.5029322334704254, 0.4700558578198995],
                         [0.29501754114337786, 1.001500704881762, 0.4637263754459029],
                         [0.34022940092323295, 1.5038138452695238, 0.46848041885940445],
                         [0.3992606828671211, 1.996387899717162, 0.47605572991991935],
                         [0.4149623712577051, 2.5035654095828046, 0.47607522521756435],
                         [0.42226599336732856, 3.0316985247334656, 0.47605059039183695],
                         [0.4855428840554379, 3.520245095637645, 0.4845695456000369] ]
                     ], 
               [0.35514002242541265, 0.38149272238117543, 0.18957523919553482, 0.05876810554464348, 
                0.012721807559782533, 0.00204739593485694, 0.00025470695859410876] )

Base.rand(rng::Distributions.AbstractRNG, d::AreaPoissonPolygon) = Base.rand(rng, __AreaP2) / d.λ^2

Distributions.pdf(d::AreaPoissonPolygon, x::Real) = Distributions.pdf(__AreaP2, d.λ^2 * x) * d.λ^2
Distributions.logpdf(d::AreaPoissonPolygon, x::Real) = Distributions.logpdf(__AreaP2, d.λ^2 * x) + 2 * log(d.λ)
Distributions.cdf(d::AreaPoissonPolygon, x::Real) = Distributions.cdf(__AreaP2, d.λ^2 * x)
Distributions.quantile(d::AreaPoissonPolygon, q::Real) = Distribution.quantile(__AreaP2, q) / d.λ^2

Base.minimum(d::AreaPoissonPolygon) = 0.0
Base.maximum(d::AreaPoissonPolygon) = Inf
Distributions.insupport(d::AreaPoissonPolygon, x::Real) = Distributions.insupport(__AreaP2, x)

Distributions.probs(d::AreaPoissonPolygon) = Distributions.probs(__AreaP2)
Distributions.component_type(d::AreaPoissonPolygon) = Distributions.component_type(__AreaP2)
function Distributions.components(d::AreaPoissonPolygon) 
    map(x -> GGamma(Distributions.scale(x) / d.λ^2, Distributions.shape(x)...), Distributions.components(__AreaP2))
end

#### Statistics

Statistics.mean(d::AreaPoissonPolygon) = Statistics.mean(__AreaP2) / d.λ^2
Statistics.var(d::AreaPoissonPolygon) = Statistics.var(__AreaP2) / d.λ^4
