"""
    GGamma(β,ν,κ)

The *Generalized gamma distribution* with scale parameter `β` and shape parameters `ν` and `κ` has probability density
function

```math
f(x; \\beta, \\nu, \\kappa) = \\frac{x^{\\nu-1} e^{-(x/\\beta)^\\kappa}}{\\Gamma(\\nu/\\kappa) \\beta^\\nu},
\\quad x > 0
```

```julia
GGamma()         # Generalized gamma distribution with unit scale and unit shapes, i.e. GGamma(1, 1, 1)
GGamma(β, ν, κ)  # Generalized gamma distribution with scale β and shapes ν and κ

params(d)        # Get the parameters, i.e. (β, ν, κ)
shape(d)         # Get the shape parameters, i.e. (ν, κ)
scale(d)         # Get the scale parameter, i.e. β
```

External links

* [Generalized gamma distribution on Wikipedia](http://en.wikipedia.org/wiki/Generalized_gamma_distribution)

"""
struct GGamma{T<:Real} <: Distributions.ContinuousUnivariateDistribution
    β::T
    ν::T
    κ::T
    GGamma{T}(β, ν, κ) where {T} = new{T}(β, ν, κ)
end

function GGamma(β::T, ν::T, κ::T; check_args::Bool=true) where {T<:Real}
    Distributions.@check_args GGamma (β, β > zero(β)) (ν, ν > zero(ν)) (κ, κ > zero(κ))
    return GGamma{T}(β, ν, κ)
end

GGamma(β::Real, ν::Real, κ::Real; check_args::Bool=true) = GGamma(promote(β, ν, κ)...; check_args=check_args)
GGamma(β::Integer, ν::Integer, κ::Integer; check_args::Bool=true) = GGamma(float(β), float(ν), float(κ); check_args=check_args)
GGamma() = GGamma{Float64}(1.0, 1.0, 1.0)

Distributions.@distr_support GGamma 0.0 Inf

#### Conversions

Distributions.convert(::Type{GGamma{T}}, β::S, ν::S, κ::S) where {T<:Real, S<:Real} = GGamma(T(β), T(ν), T(κ))
Base.convert(::Type{GGamma{T}}, d::GGamma) where {T<:Real} = GGamma{T}(T(d.β), T(d.ν), T(d.κ))
Base.convert(::Type{GGamma{T}}, d::GGamma{T}) where {T<:Real} = d

#### Parameters

Distributions.shape(d::GGamma) = (d.ν, d.κ)
Distributions.scale(d::GGamma) = d.β
Distributions.rate(d::GGamma) = 1 / d.β

Distributions.params(d::GGamma) = (d.β, d.ν, d.κ)
Distributions.partype(::GGamma{T}) where {T} = T

#### Evaluation & Sampling

Base.rand(rng::Distributions.AbstractRNG, d::GGamma) = Base.rand(rng, Distributions.Gamma(d.ν / d.κ, d.β^d.κ))^(1.0 / d.κ)

function Distributions.pdf(d::GGamma, x::Real)
    z = Distributions.pdf(Distributions.Gamma(d.ν / d.κ, d.β^d.κ), x^d.κ) * d.κ * x^(d.κ - 1)
    return x < 0 ? zero(z) : z
end

function Distributions.logpdf(d::GGamma, x::Real)
    z = Distributions.logpdf(Distributions.Gamma(d.ν / d.κ, d.β^d.κ), x^d.κ) + log(d.κ) + (d.κ - 1) * log(x)
    return x < 0 ? oftype(z, -Inf) : z
end

Distributions.cdf(d::GGamma, x::Real) = Distributions.cdf(Distributions.Gamma(d.ν / d.κ, d.β^d.κ), x^d.κ)

function Distributions.quantile(d::GGamma, q::Real)
    return d.β * Distribution.quantile(Distributions.Gamma(d.ν / d.κ, 1.0), q)^(1.0 / d.κ)
end

Base.minimum(d::GGamma) = 0.0

Base.maximum(d::GGamma) = Inf

Distributions.insupport(d::GGamma, x::Real) = Distributions.insupport(Gamma(d.ν / d.κ, d.β^d.κ), x)

#### Statistics

function moment(d::GGamma, r::Real)
    r + d.ν > 0.0 ? d.β^r * gamma((d.ν + r) / d.κ) / gamma(d.ν / d.κ) : error("This moment of GGamma does not exist")
end

Statistics.mean(d::GGamma) = moment(d, 1)

Statistics.var(d::GGamma) = moment(d, 2) - moment(d, 1)^2

function StatsBase.skewness(d::GGamma)
    μ = Statistics.mean(d)
    σ = sqrt(Statistics.var(d))
    return (moment(d, 3) - 3 * μ * σ^2 - μ^3) / σ^3
end

function StatsBase.kurtosis(d::GGamma)
    μ = Statistics.mean(d)
    σ = sqrt(Statistics.var(d))
    return (moment(d, 4) - 4 * μ * moment(d, 3) + 6 * μ^2 * σ^2 + 3 * μ^4) / σ^4
end

function StatsBase.mode(d::GGamma)
    (β, ν, κ) = Distributions.params(d)
    ν >= 1 ? β * ((ν - 1) / κ)^(1 / κ) : error("GGamma has no mode when first shape ν < 1")
end

function StatsBase.entropy(d::GGamma)
    (β, ν, κ) = Distributions.params(d)
    α = ν / κ
    α + loggamma(α) + (1 / κ - α) * digamma(α) + log(β / κ)
end
