"""
    SurfaceCroftonPolyhedron(λ)

An approximation of the distribution of the volume of the Crofton polyhedron with intensity parameter `λ` in terms of a finite mixture of generalized gamma distributions.

```julia
SurfaceCroftonPolyhedron()    # `SurfaceCroftonPolyhedron`-distribution with unit intensity, i.e. SurfaceCroftonPolyhedron(1)
SurfaceCroftonPolyhedron(λ)   # `SurfaceCroftonPolyhedron`-distribution with intensity λ
```

"""
struct SurfaceCroftonPolyhedron{T<:Real} <: Distributions.ContinuousUnivariateDistribution
    λ::T
    SurfaceCroftonPolyhedron{T}(λ) where {T} = new{T}(λ)
end

function SurfaceCroftonPolyhedron(λ::T; check_args::Bool=true) where {T<:Real}
    Distributions.@check_args SurfaceCroftonPolyhedron (λ, λ > zero(λ))
    return SurfaceCroftonPolyhedron{T}(λ)
end

SurfaceCroftonPolyhedron(λ::Integer; check_args::Bool=true) = SurfaceCroftonPolyhedron(float(λ); check_args=check_args)
SurfaceCroftonPolyhedron() = SurfaceCroftonPolyhedron{Float64}(1.0)

Distributions.@distr_support SurfaceCroftonPolyhedron 0.0 Inf

#### Conversions

Distributions.convert(::Type{SurfaceCroftonPolyhedron{T}}, λ::S) where {T<:Real, S<:Real} = SurfaceCroftonPolyhedron(T(λ))
Base.convert(::Type{SurfaceCroftonPolyhedron{T}}, d::SurfaceCroftonPolyhedron) where {T<:Real} = SurfaceCroftonPolyhedron{T}(T(d.λ))
Base.convert(::Type{SurfaceCroftonPolyhedron{T}}, d::SurfaceCroftonPolyhedron{T}) where {T<:Real} = d

#### Parameters

Distributions.params(d::SurfaceCroftonPolyhedron) = d.λ
Distributions.partype(::SurfaceCroftonPolyhedron{T}) where {T} = T

#### Evaluation & Sampling

__SurfaceC3 = Distributions.MixtureModel( 
               GGamma[GGamma(pars...) for pars in 
                       [ [1.2836216833005776, 2.0, 0.47783146688132433],
                         [1.483584338294107, 2.5, 0.4793288227930474],
                         [1.7010473117911977, 3.0, 0.4845453110646308],
                         [1.8392116211713576, 3.5, 0.486924274642886],
                         [1.9525542889141085, 4.0, 0.4888677103774066],
                         [2.0363052307350586, 4.5, 0.49006259992824314],
                         [2.12114635164391, 5.0, 0.4915079667549333],
                         [2.1706560548158604, 5.5, 0.49190451013724074],
                         [2.220935418969108, 6.0, 0.49252187945102366],
                         [2.298803873745069, 6.5, 0.4940537927090746],
                         [2.3324263861995362, 7.0, 0.49423647053228104],
                         [2.3434563118921687, 7.5, 0.49393203200934394],
                         [2.518165350046965, 8.0, 0.49811183663834907],
                         [2.3607744384658846, 8.5, 0.4935303292990052],
                         [2.927973678393712, 9.0, 0.5062786997755073],
                         [2.035669463804619, 9.5, 0.4848280983589908] ]
                     ], 
               [0.004821854486955703, 0.03649691241511029, 0.10294136323740459, 0.17171264035283598, 
                0.2018417808121232, 0.18293356714930878, 0.1347236223769329, 0.08377204662412686,
                0.04510100964140895, 0.021305440751480494, 0.009076102559958929, 0.003483939368514864,
                0.0012397140087682993, 0.00039280443869015724, 0.00012180137635555282, 0.000035400400024520276] )

Base.rand(rng::Distributions.AbstractRNG, d::SurfaceCroftonPolyhedron) = Base.rand(rng, __SurfaceC3) / d.λ^2

Distributions.pdf(d::SurfaceCroftonPolyhedron, x::Real) = Distributions.pdf(__SurfaceC3, d.λ^2 * x) * d.λ^2
Distributions.logpdf(d::SurfaceCroftonPolyhedron, x::Real) = Distributions.logpdf(__SurfaceC3, d.λ^2 * x) + 2 * log(d.λ)
Distributions.cdf(d::SurfaceCroftonPolyhedron, x::Real) = Distributions.cdf(__SurfaceC3, d.λ^2 * x)
Distributions.quantile(d::SurfaceCroftonPolyhedron, q::Real) = Distribution.quantile(__SurfaceC3, q) / d.λ^2

Base.minimum(d::SurfaceCroftonPolyhedron) = 0.0
Base.maximum(d::SurfaceCroftonPolyhedron) = Inf
Distributions.insupport(d::SurfaceCroftonPolyhedron, x::Real) = Distributions.insupport(__SurfaceC3, x)

Distributions.probs(d::SurfaceCroftonPolyhedron) = Distributions.probs(__SurfaceC3)
Distributions.component_type(d::SurfaceCroftonPolyhedron) = Distributions.component_type(__SurfaceC3)
function Distributions.components(d::SurfaceCroftonPolyhedron) 
    map(x -> GGamma(Distributions.scale(x) / d.λ^2, Distributions.shape(x)...), Distributions.components(__SurfaceC3))
end

#### Statistics

Statistics.mean(d::SurfaceCroftonPolyhedron) = Statistics.mean(__SurfaceC3) / d.λ^2
Statistics.var(d::SurfaceCroftonPolyhedron) = Statistics.var(__SurfaceC3) / d.λ^4
