"""
    SurfacePoissonPolyhedron(λ)

An approximation of the distribution of the volume of the Poisson polyhedron with intensity parameter `λ` in terms of a finite mixture of generalized gamma distributions.

```julia
SurfacePoissonPolyhedron()    # `SurfacePoissonPolyhedron`-distribution with unit intensity, i.e. SurfacePoissonPolyhedron(1)
SurfacePoissonPolyhedron(λ)   # `SurfacePoissonPolyhedron`-distribution with intensity λ
```

"""
struct SurfacePoissonPolyhedron{T<:Real} <: Distributions.ContinuousUnivariateDistribution
    λ::T
    SurfacePoissonPolyhedron{T}(λ) where {T} = new{T}(λ)
end

function SurfacePoissonPolyhedron(λ::T; check_args::Bool=true) where {T<:Real}
    Distributions.@check_args SurfacePoissonPolyhedron (λ, λ > zero(λ))
    return SurfacePoissonPolyhedron{T}(λ)
end

SurfacePoissonPolyhedron(λ::Integer; check_args::Bool=true) = SurfacePoissonPolyhedron(float(λ); check_args=check_args)
SurfacePoissonPolyhedron() = SurfacePoissonPolyhedron{Float64}(1.0)

Distributions.@distr_support SurfacePoissonPolyhedron 0.0 Inf

#### Conversions

Distributions.convert(::Type{SurfacePoissonPolyhedron{T}}, λ::S) where {T<:Real, S<:Real} = SurfacePoissonPolyhedron(T(λ))
Base.convert(::Type{SurfacePoissonPolyhedron{T}}, d::SurfacePoissonPolyhedron) where {T<:Real} = SurfacePoissonPolyhedron{T}(T(d.λ))
Base.convert(::Type{SurfacePoissonPolyhedron{T}}, d::SurfacePoissonPolyhedron{T}) where {T<:Real} = d

#### Parameters

Distributions.params(d::SurfacePoissonPolyhedron) = d.λ
Distributions.partype(::SurfacePoissonPolyhedron{T}) where {T} = T

#### Evaluation & Sampling

__SurfaceP3 = Distributions.MixtureModel( 
               GGamma[GGamma(pars...) for pars in 
                       [ [1.2098556062869072, 0.5, 0.47631123628916733],
                         [1.3412751175727748, 1.0, 0.4734821871989209],
                         [1.5667073059329948, 1.5, 0.4798785725245006],
                         [1.7567573783157748, 2.0, 0.4848448613977591],
                         [1.8941650317146754, 2.5, 0.4877345361843266],
                         [1.9981763817835565, 3.0, 0.4895364585922699],
                         [2.076691099918641, 3.5, 0.4906692571148067],
                         [2.140765125176424, 4.0, 0.491525549516008],
                         [2.228957634222716, 4.5, 0.49332516717839375],
                         [2.274467690200302, 5.0, 0.4935910239654529],
                         [2.1511255425361338, 5.5, 0.4893375141924883],
                         [2.5553436718961087, 6.0, 0.5007135763069473],
                         [1.694447952558398, 6.5, 0.4726907865207424] ]
                     ], 
               [0.18804953961613277, 0.2582798295505683, 0.2254223535477698, 0.15632271311079013, 
                0.09150472863972058, 0.046446770152869285, 0.02090691561809119, 0.008437230872739333,
                0.003121326219140241, 0.0010568288773625699, 0.0003305427765593231, 0.00009630080892679498,
                0.00002492020932975837] )

Base.rand(rng::Distributions.AbstractRNG, d::SurfacePoissonPolyhedron) = Base.rand(rng, __SurfaceP3) / d.λ^2

Distributions.pdf(d::SurfacePoissonPolyhedron, x::Real) = Distributions.pdf(__SurfaceP3, d.λ^2 * x) * d.λ^2
Distributions.logpdf(d::SurfacePoissonPolyhedron, x::Real) = Distributions.logpdf(__SurfaceP3, d.λ^2 * x) + 2 * log(d.λ)
Distributions.cdf(d::SurfacePoissonPolyhedron, x::Real) = Distributions.cdf(__SurfaceP3, d.λ^2 * x)
Distributions.quantile(d::SurfacePoissonPolyhedron, q::Real) = Distribution.quantile(__SurfaceP3, q) / d.λ^2

Base.minimum(d::SurfacePoissonPolyhedron) = 0.0
Base.maximum(d::SurfacePoissonPolyhedron) = Inf
Distributions.insupport(d::SurfacePoissonPolyhedron, x::Real) = Distributions.insupport(__SurfaceP3, x)

Distributions.probs(d::SurfacePoissonPolyhedron) = Distributions.probs(__SurfaceP3)
Distributions.component_type(d::SurfacePoissonPolyhedron) = Distributions.component_type(__SurfaceP3)
function Distributions.components(d::SurfacePoissonPolyhedron) 
    map(x -> GGamma(Distributions.scale(x) / d.λ^2, Distributions.shape(x)...), Distributions.components(__SurfaceP3))
end

#### Statistics

Statistics.mean(d::SurfacePoissonPolyhedron) = Statistics.mean(__SurfaceP3) / d.λ^2
Statistics.var(d::SurfacePoissonPolyhedron) = Statistics.var(__SurfaceP3) / d.λ^4
