"""
    VolumePoissonPolyhedron(λ)

An approximation of the distribution of the volume of the Poisson polyhedron with intensity parameter `λ` in terms of a finite mixture of generalized gamma distributions.

```julia
VolumePoissonPolyhedron()    # `VolumePoissonPolyhedron`-distribution with unit intensity, i.e. VolumePoissonPolyhedron(1)
VolumePoissonPolyhedron(λ)   # `VolumePoissonPolyhedron`-distribution with intensity λ
```

"""
struct VolumePoissonPolyhedron{T<:Real} <: Distributions.ContinuousUnivariateDistribution
    λ::T
    VolumePoissonPolyhedron{T}(λ) where {T} = new{T}(λ)
end

function VolumePoissonPolyhedron(λ::T; check_args::Bool=true) where {T<:Real}
    Distributions.@check_args VolumePoissonPolyhedron (λ, λ > zero(λ))
    return VolumePoissonPolyhedron{T}(λ)
end

VolumePoissonPolyhedron(λ::Integer; check_args::Bool=true) = VolumePoissonPolyhedron(float(λ); check_args=check_args)
VolumePoissonPolyhedron() = VolumePoissonPolyhedron{Float64}(1.0)

Distributions.@distr_support VolumePoissonPolyhedron 0.0 Inf

#### Conversions

Distributions.convert(::Type{VolumePoissonPolyhedron{T}}, λ::S) where {T<:Real, S<:Real} = VolumePoissonPolyhedron(T(λ))
Base.convert(::Type{VolumePoissonPolyhedron{T}}, d::VolumePoissonPolyhedron) where {T<:Real} = VolumePoissonPolyhedron{T}(T(d.λ))
Base.convert(::Type{VolumePoissonPolyhedron{T}}, d::VolumePoissonPolyhedron{T}) where {T<:Real} = d

#### Parameters

Distributions.params(d::VolumePoissonPolyhedron) = d.λ
Distributions.partype(::VolumePoissonPolyhedron{T}) where {T} = T

#### Evaluation & Sampling

__VolumeP3 = Distributions.MixtureModel( 
               GGamma[GGamma(pars...) for pars in 
                       [ [0.033681241902021, 0.3355508842426562, 0.30008307849377275],
                         [0.04113615419408623, 0.6595766270725966, 0.2981908025642593],
                         [0.05299472537895758, 0.9927488925585066, 0.30150394255716034],
                         [0.06147752161536205, 1.334492209624727, 0.3031240979165031],
                         [0.07121148183781656, 1.67255899102764, 0.3052170640360764],
                         [0.0915920344421473, 1.9915750548297244, 0.3101842815284165],
                         [0.08770292276633974, 2.3528725982726364, 0.30816346431336145],
                         [0.10518409956707088, 2.672694101609369, 0.31162787711767065],
                         [0.12674263541746703, 2.989283198876097, 0.3153634767127447],
                         [0.10056625430449358, 3.389842268253903, 0.30939174263955727],
                         [0.030030870547503846, 4.021770356880307, 0.2856185871123655],
                         [1.195679597506279, 3.4529018073411346, 0.37525070779746666],
                         [0.18154188582843386, 4.139972114170055, 0.31812045399857886] ]
                     ], 
               [0.18804953961613277, 0.2582798295505683, 0.2254223535477698, 0.15632271311079013, 
                0.09150472863972058, 0.046446770152869285, 0.02090691561809119, 0.008437230872739333,
                0.003121326219140241, 0.0010568288773625699, 0.0003305427765593231, 0.00009630080892679498,
                0.00002492020932975837] )

Base.rand(rng::Distributions.AbstractRNG, d::VolumePoissonPolyhedron) = Base.rand(rng, __VolumeP3) / d.λ^3

Distributions.pdf(d::VolumePoissonPolyhedron, x::Real) = Distributions.pdf(__VolumeP3, d.λ^3 * x) * d.λ^3
Distributions.logpdf(d::VolumePoissonPolyhedron, x::Real) = Distributions.logpdf(__VolumeP3, d.λ^3 * x) + 3 * log(d.λ)
Distributions.cdf(d::VolumePoissonPolyhedron, x::Real) = Distributions.cdf(__VolumeP3, d.λ^3 * x)
Distributions.quantile(d::VolumePoissonPolyhedron, q::Real) = Distribution.quantile(__VolumeP3, q) / d.λ^3

Base.minimum(d::VolumePoissonPolyhedron) = 0.0
Base.maximum(d::VolumePoissonPolyhedron) = Inf
Distributions.insupport(d::VolumePoissonPolyhedron, x::Real) = Distributions.insupport(__VolumeP3, x)

Distributions.probs(d::VolumePoissonPolyhedron) = Distributions.probs(__VolumeP3)
Distributions.component_type(d::VolumePoissonPolyhedron) = Distributions.component_type(__VolumeP3)
function Distributions.components(d::VolumePoissonPolyhedron) 
    map(x -> GGamma(Distributions.scale(x) / d.λ^3, Distributions.shape(x)...), Distributions.components(__VolumeP3))
end

#### Statistics

Statistics.mean(d::VolumePoissonPolyhedron) = Statistics.mean(__VolumeP3) / d.λ^3
Statistics.var(d::VolumePoissonPolyhedron) = Statistics.var(__VolumeP3) / d.λ^6
